/* Copyright (C) 2005, 2006 Hartmut Holzgraefe, MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdlib.h>

#include <ndb_init.h>
#include <mgmapi.h>
#include <mgmapi_debug.h>
#include <ndb_logevent.h>

#define DBTUP 0xf8 // DBTUP block reports data usage
#define DBTUX 0xf9 // DBTUX block reports index usage

#define CONNECT_STRING "localhost"

int main(int argc, char **argv)
{
  NdbMgmHandle handle;
  struct ndb_mgm_cluster_state *cluster_state;
  struct ndb_mgm_node_state    *node_state;

  ndb_init();

  // creating connection handle, setting conncet string
  handle = ndb_mgm_create_handle();
  ndb_mgm_set_connectstring(handle, CONNECT_STRING);

 
  // establish connections
  if (ndb_mgm_connect(handle, 5, 1, 1) < 0) {
	  ndb_mgm_destroy_handle(&handle);
	  exit(3);
  }
  cluster_state = ndb_mgm_get_status(handle);
  if (cluster_state == NULL) {
	ndb_mgm_disconnect(handle);
	ndb_mgm_destroy_handle(&handle);
	exit(3);
  }

  // MemoryUsage events are issued in the statistics category at log level 5
  // so we filter out anything not statistics related or at a log level 6-15
  int filter[] = { 5, NDB_MGM_EVENT_CATEGORY_STATISTIC,
				   0 };

  struct ndb_logevent_handle * le_handle= ndb_mgm_create_logevent_handle(handle, filter);
  struct ndb_logevent le;
  int status;
  
  int dump_code = 1000;
  struct ndb_mgm_reply reply;

  ndb_mgm_dump_state(handle, 2, &dump_code, 1, &reply);
  ndb_mgm_dump_state(handle, 3, &dump_code, 1, &reply);

  // event loop
  do {
	status = ndb_logevent_get_next(le_handle, &le, 1000 /* 1 sec timeout */);

	if (status == 1) { // event received successfully, no timeout

	  switch (le.type) { // handle events by type

		// NDB_LE_MemoryUsage events carry the ALL DUMP 1000 results
	    case NDB_LE_MemoryUsage:
		  printf("Time %u  Node %2d  ", le.time, le.source_nodeid);
		  switch (le.MemoryUsage.block) {
		    case DBTUP: 
		      printf("Data  ");
		      break;
		    case DBTUX:
		      printf("Index ");
		      break;
		  }
		  printf(" %4u of %4u %6uKB pages\n"
			     , le.MemoryUsage.pages_used
			     , le.MemoryUsage.pages_total
			     , le.MemoryUsage.page_size_kb);
		  break;
		
	    default:
		  break;
	  }

	}

  } while (status >= 0);

  printf("Ended with status %d\n", status);

  exit(0);
}
